@@ -23,7 +23,6 @@ from Orange.OrangeWidgets.OWToolbars import ZoomSelectToolbar
 from Orange.OrangeWidgets.OWWidget import *
 
 from ..obiExpression import *
-from ..obiGEO import transpose
 
 
 NAME = "Differential expression"
@@ -85,8 +84,7 @@ class ScoreHist(OWInteractiveHist):
 
     def setBoundary(self, low, hi):
         OWInteractiveHist.setBoundary(self, low, hi)
-        self.master.update_selected_info_label(low, hi)
-        self.master.commit_if()
+        self.master.on_boundary_change(low, hi)
 
 
 def disable_controls(method):
@@ -107,20 +105,25 @@ from Orange.orng.orngDataCaching import data_hints
 from .OWGenotypeDistances import SetContextHandler
 from .OWVulcanoPlot import LabelSelectionWidget
 
+
 class OWFeatureSelection(OWWidget):
-    settingsList = ["method_index", "dataLabelIndex", "compute_null",
-                    "permutations_count", "selectPValue", "auto_commit"]
-    contextHandlers={"Data": DomainContextHandler("Data", ["genes_in_columns"]),
-                     "TargetSelection": SetContextHandler("TargetSelection",
-                                                          findImperfect=False)}
-    
+    settingsList = [
+        "method_index", "dataLabelIndex", "compute_null",
+        "permutations_count", "selectPValue", "auto_commit",
+        "thresholds"]
+
+    contextHandlers = {
+        "Data": DomainContextHandler("Data", ["genes_in_columns"]),
+        "TargetSelection":
+            SetContextHandler("TargetSelection", findImperfect=False)
+    }
+
     def __init__(self, parent=None, signalManager=None, name="Gene selection"):
         OWWidget.__init__(self, parent, signalManager, name, wantGraph=True, showSaveGraph=True)
         self.inputs = [("Examples", ExampleTable, self.set_data)]
         self.outputs = [("Example table with selected genes", ExampleTable), ("Example table with remaining genes", ExampleTable), ("Selected genes", ExampleTable)]
 
         self.method_index = 0
-#        self.dataLabelIndex = 0
         self.genes_in_columns = False
         self.compute_null = False
         self.permutations_count = 10
@@ -129,6 +132,12 @@ class OWFeatureSelection(OWWidget):
         self.selectPValue = 0.01
         self.data_changed_flag = False
         self.add_scores_to_output = True
+        self.thresholds = {
+            "fold change": (0.5, 2.),
+            "log2 fold change": (-1, 1),
+            "t-test": (-2, 2),
+            "t-test p-value": (0.01, 0.01),
+        }
 
         self.oneTailTestHi = oneTailTestHi = lambda array, low, hi: array >= hi
         self.oneTailTestLow = oneTailTestLow = lambda array, low, hi: array <= low
@@ -179,25 +188,20 @@ class OWFeatureSelection(OWWidget):
         self.genes_in_columns_check = OWGUI.checkBox(box, self, "genes_in_columns",
                                                   "Genes in columns",
                                                   callback=self.on_genes_in_columns_change)
-    
-#        ZoomSelectToolbar(self, self.controlArea, self.histogram,
-#                          buttons=[ZoomSelectToolbar.IconSelect,
-#                                   ZoomSelectToolbar.IconZoom,
-#                                   ZoomSelectToolbar.IconPan])
-        
+
         box = OWGUI.widgetBox(self.controlArea, "Selection")
         box.layout().setSpacing(0)
-        callback = self.update_boundary
+
         self.upperBoundarySpin = OWGUI.doubleSpin(box, self, "histogram.upperBoundary",
                                                   min=-1e6, max=1e6, step= 1e-6,
                                                   label="Upper threshold:", 
-                                                  callback=callback, 
+                                                  callback=self.update_boundary, 
                                                   callbackOnReturn=True)
         
         self.lowerBoundarySpin = OWGUI.doubleSpin(box, self, "histogram.lowerBoundary", 
                                                   min=-1e6, max=1e6, step= 1e-6, 
                                                   label="Lower threshold:", 
-                                                  callback=callback, 
+                                                  callback=self.update_boundary, 
                                                   callbackOnReturn=True)
         
         check = OWGUI.checkBox(box, self, "compute_null", "Compute null distribution",
@@ -236,7 +240,6 @@ class OWFeatureSelection(OWWidget):
         self.scoreCache = {}
         self.nullDistCache = {}
         self.cuts = {}
-        self.discretizer = orange.EquiNDiscretization(numberOfIntervals=5)
         self.null_dist = []
         self.targets = []
         self.scores = {}
@@ -409,32 +412,47 @@ class OWFeatureSelection(OWWidget):
         else:
             self.null_dist = []
         pb.advance()
-        self.histogram.type = self.histType[self.score_methods[self.method_index][2]]
+        htype = self.histType[self.score_methods[self.method_index][2]]
+        score_type = self.score_methods[self.method_index][0]
+        self.histogram.type = htype
         if self.scores:
             self.histogram.setValues(self.scores.values())
-            self.histogram.setBoundary(self.histogram.minx if self.histogram.type in ["lowTail", "twoTail"] else self.histogram.maxx,
-                                       self.histogram.maxx if self.histogram.type in ["hiTail", "twoTail"] else self.histogram.minx)
+            low, high = self.thresholds.get(score_type, (float("-inf"), float("inf")))
+            minx, maxx = self.histogram.minx, self.histogram.maxx
+            low, high = max(low, minx), min(high, maxx)
+
+            if htype == "hiTail":
+                low = high
+            if htype == "lowTail":
+                high = low
+
+            self.histogram.setBoundary(low, high)
+
             if self.compute_null and self.null_dist:
                 nullY, nullX = numpy.histogram(self.null_dist, bins=self.histogram.xData)
-                self.histogram.nullCurve = self.histogram.addCurve("nullCurve",
-                        Qt.black, Qt.black, 6, symbol=QwtSymbol.NoSymbol,
-                        style=QwtPlotCurve.Steps, xData = nullX,
-                        yData = nullY/self.permutations_count)
-                
-                minx = min(min(nullX), self.histogram.minx)
-                maxx = max(max(nullX), self.histogram.maxx)
-                miny = min(min(nullY/self.permutations_count), self.histogram.miny)
-                maxy = max(max(nullY/self.permutations_count), self.histogram.maxy)
+                nullY = nullY / self.permutations_count
+                self.histogram.nullCurve = self.histogram.addCurve(
+                    "nullCurve", Qt.black, Qt.black, 6,
+                    symbol=QwtSymbol.NoSymbol, style=QwtPlotCurve.Steps,
+                    xData=nullX, yData=nullY)
+
+                minx = min(min(nullX), minx)
+                maxx = max(max(nullX), maxx)
+                miny = min(min(nullY), self.histogram.miny)
+                maxy = max(max(nullY), self.histogram.maxy)
+                spanx, spany = maxx - minx, maxy - miny
+                self.histogram.setAxisScale(
+                    QwtPlot.xBottom, minx - 0.05 * spanx, maxx + 0.05 * spanx)
+                self.histogram.setAxisScale(
+                    QwtPlot.yLeft, miny - 0.05 * spany, maxy + 0.05 * spany)
 
-                self.histogram.setAxisScale(QwtPlot.xBottom, minx - (0.05 * (maxx - minx)), maxx + (0.05 * (maxx - minx)))
-                self.histogram.setAxisScale(QwtPlot.yLeft, miny - (0.05 * (maxy - miny)), maxy + (0.05 * (maxy - miny)))
             state = dict(hiTail=(False, True), lowTail=(True, False), twoTail=(True, True))
             for spin, visible in zip((self.upperBoundarySpin, self.lowerBoundarySpin), state[self.histogram.type]):
                 spin.setVisible(visible)
-            
+
             # If this is a two sample test add markers to the left and right
             # plot indicating which target group is over-expressed in that
-            # part  
+            # part
             if self.method_index in [0, 2, 6]:
                 if self.method_index == 0: ## fold change is centered on 1.0
                     x1, y1 = (self.histogram.minx + 1) / 2 , self.histogram.maxy
@@ -462,7 +480,13 @@ class OWFeatureSelection(OWWidget):
         pb.advance()
         pb.finish()
         self.update_data_info_label()
-            
+
+    def on_boundary_change(self, low, high):
+        stype = self.score_methods[self.method_index][0]
+        self.thresholds[stype] = (low, high)
+        self.update_selected_info_label(low, high)
+        self.commit_if()
+
     def update_data_info_label(self):
         if self.data:
             samples, genes = len(self.data), len(self.data.domain.attributes)
@@ -569,8 +593,11 @@ class OWFeatureSelection(OWWidget):
         if self.data and self.genes_in_columns:
             selected = sorted(selected)
             if selected:
-                newdata = orange.ExampleTable(orange.Domain(self.data.domain),
-                                        [self.data[int(i)] for i in selected])
+                newdata = orange.ExampleTable(
+                    orange.Domain(self.data.domain),
+                    [self.data[int(i)] for i in selected],
+                    name=self.data.name
+                )
             else:
                 newdata = None
             if self.add_scores_to_output:
@@ -586,7 +613,11 @@ class OWFeatureSelection(OWWidget):
             
             remaining = sorted(remaining)
             if remaining:
-                newdata = orange.ExampleTable(orange.Domain(self.data.domain), [self.data[int(i)] for i in remaining])
+                newdata = orange.ExampleTable(
+                    orange.Domain(self.data.domain),
+                    [self.data[int(i)] for i in remaining],
+                    name=self.data.name
+                )
             else:
                 newdata = None
             
@@ -598,29 +629,42 @@ class OWFeatureSelection(OWWidget):
             self.send("Example table with remaining genes", newdata)
             
         elif self.data and not self.genes_in_columns:
-            
-            selected_attrs = [attr for attr in self.data.domain.attributes \
-                             if attr in selected or \
-                                attr.varType == orange.VarTypes.String]
+            method_name = self.score_methods[self.method_index][0]
+            selected_attrs = [attr for attr in self.data.domain.attributes
+                              if attr in selected or
+                              attr.varType == orange.VarTypes.String]  # ?? why strings
+            if self.add_scores_to_output:
+                scores = [self.scores[attr] for attr in selected_attrs]
+                attrs = [copy_descriptor(attr) for attr in selected_attrs]
+                for attr, score in zip(attrs, scores):
+                    attr.attributes[method_name] = str(score)
+                selected_attrs = attrs
+
             newdomain = orange.Domain(selected_attrs, self.data.domain.classVar)
-            if self.add_scores_to_output:
-                for attr in newdomain.attributes:
-                    attr.attributes[self.score_methods[self.method_index][0]] = str(self.scores[attr])
             newdomain.addmetas(self.data.domain.getmetas())
-            newdata = orange.ExampleTable(newdomain, self.data)
-            
-            self.send("Example table with selected genes", newdata if selected_attrs else None)
-            
-            remaining_attrs = [attr for attr in self.data.domain.attributes if attr in remaining]
+            newdata = orange.ExampleTable(
+                newdomain, self.data, name=self.data.name
+            )
+            self.send("Example table with selected genes",
+                      newdata if selected_attrs else None)
+
+            remaining_attrs = [attr for attr in self.data.domain.attributes
+                               if attr in remaining]
+            if self.add_scores_to_output:
+                scores = [self.scores[attr] for attr in remaining_attrs]
+                attrs = [copy_descriptor(attr) for attr in remaining_attrs]
+                for attr, score in zip(attrs, scores):
+                    attr.attributes[method_name] = str(scores)
+                remaining_attrs = attrs
+
             newdomain = orange.Domain(remaining_attrs, self.data.domain.classVar)
-            if self.add_scores_to_output:
-                for attr in newdomain.attributes:
-                    attr.attributes[self.score_methods[self.method_index][0]] = str(self.scores[attr])
             newdomain.addmetas(self.data.domain.getmetas())
-            newdata = orange.ExampleTable(newdomain, self.data)
-            
-            self.send("Example table with remaining genes", newdata if remaining_attrs else None)
-            
+            newdata = orange.ExampleTable(
+                newdomain, self.data, name=self.data.name
+            )
+            self.send("Example table with remaining genes",
+                      newdata if remaining_attrs else None)
+
             domain = orange.Domain([orange.StringVariable("label"),
                                     orange.FloatVariable(self.score_methods[self.method_index][0])],
                                     False)
@@ -686,17 +730,23 @@ class OWFeatureSelection(OWWidget):
     def settingsToWidgetCallbackTargetSelection(self, handler, context):
         self.target_selections = getattr(context, "target_selections", self.target_selections)
         self.current_target_selection = getattr(context, "current_target_selection", self.current_target_selection)
-    
+
+
+def copy_descriptor(desc):
+    d = desc.clone()
+    d.attributes = dict(desc.attributes)
+    c = Orange.core.ClassifierFromVar()
+    c.whichVar = desc
+    d.get_value_from = c
+    return d
+
+
 if __name__=="__main__":
     import sys
     app = QApplication(sys.argv)
-#    data = orange.ExampleTable("/home/marko/t2.tab")
-#    data = orange.ExampleTable("../../../doc/datasets/brown-selected")
     data = orange.ExampleTable(os.path.expanduser("~/Documents/GDS636"))
     w = OWFeatureSelection()
     w.show()
     w.set_data(data)
-#    w.set_data(None)
-#    w.set_data(data)
     app.exec_()
     w.saveSettings()
